%---------------------------------------------------------
%
%   SPA_ARMA_KIKA_STAT.M
%
%   This function provides an ARMA spectral estimate Ki-Ka
%   based on cepstrum and impulsional response, derived the
%   first from the periodogram
% The routine requires:
%
%   xin(nsam)   Input signal (real valued) of nsam samples
%   hmod(nres)  Actual spectral dessity with nres the
%               resolution for frequency plot.
%   stip        String including data of the actual ARMA model
%               to be depicted in the plot title
%   nq          Order of the denominator (# of poles)
%   np          Order of the numerator (# of ceros)
%
% The routine provides:
%
%   outd(np)    The numerator coefficients outn(1)=1
%   outn(nq)    The denominator coefficients outd(1)=1
%   sigma       The input noise power
%
%   Miguel Angel Lagunas                July 2007
%
%---------------------------------------------------------------
function [sigma,outn,outd]=spa_arma_per_stat(xinf,hmod,himp,stip,np,nq);
nres=length(hmod);nsam=length(xinf);
% Compute cepstrum
% Determines the size of the impulse response
mm1=128;nq1=nq-1;np1=np-1;
% Number of samples per record for periodogram
nsam4=max([nq1 np1 nsam/2]);
nns=2;
while nns<nsam4;
   nns=2*nns;
end;
stp=['Periodogram for constrains'];
% Routine SPA_WOS_CEPS
xb=spa_wos_ceps(xinf,nns,nns,stp);
% Compute cepstrum
disp(nns);
cuso=ifft(log(xb));
% Choose length
mm1=30;
% Compute in cep(.) the coefficients of cepstrum deriv.
cep=(1:1:mm1-1).*cuso(2:mm1);clear cuso;
% Compute impulse response
i=2;h(1)=1;
while i<mm1+0.5;
   h(i)=0;
   for k=1:i-1;
       h(i)=h(i)+h(k)*cep(i-k);
   end;
   h(i)=h(i)/(i-1);
   i=i+1;
end;
figure(7);plot(h(2:mm1),'g');hold on;plot(himp(2:mm1),'r');hold on;
% Write the system matrix
hh=zeros(mm1,nq);
for i=1:mm1;
    for k=1:nq;
        if k<=i;
            hh(i,k)=h(i-k+1);
        end;
    end;
end;
cuso=hh(nq+1:mm1,2:nq);duso=hh(nq+1:mm1,1);
outd=-(inv(cuso'*cuso))*(cuso'*duso);
outn=hh(2:nq,1)+hh(2:nq,2:nq)*outd;
outd=[1;outd]';outn=[1;outn]';sigma=1;
sigma2=sigma^2;
disp('Noise power');disp(sigma2);
disp('Numerator coefficients');disp(outn);
disp('Coefficients denorminator');disp(outd);
%---------------------------------------------------------------------