%---------------------------------------------------------
%
%   SPA_ARMA_PER_STAT.M
%
%   This function provides an ARMA spectral estimate which is maximum
% entropy satisfying correlation constraint (number of poles)
% and cepstrum constrains (number of ceros) both from the periodogram
% The routine requires:
%
%   xin(nsam)   Input signal (real valued) of nsam samples
%   hmod(nres)  Actual spectral dessity with nres the
%               resolution for frequency plot.
%   stip        String including data of the actual ARMA model
%               to be depicted in the plot title
%   nq          Order of the denominator (# of poles)
%   np          Order of the numerator (# of ceros)
%
% The routine provides:
%
%   outd(np)    The numerator coefficients outn(1)=1
%   outn(nq)    The denominator coefficients outd(1)=1
%   sigma       The input noise power
%
%   Miguel Angel Lagunas                June 2007
%
%---------------------------------------------------------------
function [sigma,outn,outd,np,nq]=spa_arma_per_stat(xinf,hmod,himp,stip,np,nq);
    nres=length(hmod);nsam=length(xinf);
    % Computes autocorrelation matrix ra(nq,nq).Routine SPA_COR.M
    ra1=spa_corc(xinf,nq);
    % Compute cepstrum
    % Determines the size of the starting correlation matrix, to be used
    % to compute cepstrum.
    nle=(nsam/2);
    ra2=spa_corc(xinf,nle);jj=sqrt(-1);
    % Compute frequency cepstrum
    nle2=nle;
    for i=1:nsam;
        sc=exp(jj*2*pi*(i-1)*(0:1:(nle2-1))'/nsam);
        per(i)=real(sc'*ra2*sc/(nle2));
    end;
    % Compute time cepstrum
    auso=real(ifft(log(per),nsam));cuso=auso(2:np+1);    
    cep=(1:1:np).*cuso(1:np);clear cuso auso;
    % Compute correlation
    auso=real(ifft(per));cuso=auso(1:nq);ra=toeplitz(cuso);
    clear auso cuso;
    % Compute impulse response
    i=2;h(1)=1;
    while i<np+0.5;
       h(i)=0;
       for k=1:i-1;
          h(i)=h(i)+h(k)*cep(i-k);
       end;
       h(i)=h(i)/(i-1);i=i+1;
     end;
     figure(7);plot(h(2:np),'g');hold on;plot(himp(2:np),'r');hold on;
     ha=h.*exp(-(0:1:np-1)/(0.1*np));ha=h;
     h(2:np)=himp(2:np);
     % Write the system Haenkel matrix
     hh=zeros(np,nq);
     for i=1:np;
       for k=1:nq;
         if k<=i;
            hh(i,k)=ha(i-k+1);
         end;
       end;
     end;
     % Find minimum eigenvector
     [u d]=eig(ra1,hh'*hh);
     sas=diag(d);ua=[];sasn=[];
     for j=1:nq;
       dmin=10e6;
       for i=1:nq;
         if abs(sas(i))<dmin;
           dmin=abs(sas(i));nqq=i;
         end;
       end;
       sasn=[sasn;sas(nqq)];sas(nqq)=10e9;ua=[ua u(:,nqq)];
     end;
     % Denominator coefficients
     outd=real(ua(:,1)');
     % For overestimated order
     iover=0;
     if iover==1;
       sad=diff(sasn);ila=1;
       for i=1:length(sad);
         if sad(i)<1.5;
             outd=outd+ua(:,i+1)';ila=ila+1;
         else;
             break;
         end;
       end;
     end;
     outd=outd/outd(1);sigma=1/sqrt(sasn(1));
     hha=hh/sigma;
     % Numerator coefficients
     outn=outd*hha';
     % Normalization
     sigma=sigma*outn(1);outn=outn/outn(1);
     %%pause;
%---------------------------------------------------------------------