%---------------------------------------------------------
%
%   SPA_ARMA_PISMI_STAT.M
%
%   This function provides an ARMA spectral estimate which is maximum
% entropy satisfying correlation constraint (number of poles)
% and cepstrum constrains (number of ceros) both from the periodogram
% The routine requires:
%
%   xin(nsam)   Input signal (real valued) of nsam samples
%   hmod(nres)  Actual spectral dessity with nres the
%               resolution for frequency plot.
%   stip        String including data of the actual ARMA model
%               to be depicted in the plot title
%   nq          Order of the denominator (# of poles)
%   np          Order of the numerator (# of ceros)
%
% The routine provides:
%
%   outd(np)    The numerator coefficients outn(1)=1
%   outn(nq)    The denominator coefficients outd(1)=1
%   sigma       The input noise power
%
%   Miguel Angel Lagunas                June 2007
%
%---------------------------------------------------------------
function [sigma,outn,outd,np,nq,leigen]=spa_arma_pismi(xinf,hmod,himp,stip,np,nq);
    nres=length(hmod);nsam=length(xinf);
    % Computes autocorrelation matrix ra(nq,nq).Routine SPA_COR.M
    ra1=spa_corc(xinf,nq);
    % Compute cepstrum
    % Determines the size of the starting correlation matrix, to be used
    % to compute cepstrum.
    nle=(nsam/2);
    %nle=max([nsam-10*np nsam/2]);
    ra2=spa_corc(xinf,nle);
    % Eigendecomposition of the correlation matrix for cepstrum
    [u d]=eig(ra2);
    % Pisarenko framework for cepstrum estimation
    d1=diag(log(diag(d)));clear d;leigen=diag(d1);
    ca1=u*d1*u';clear ra2 u d1;jj=sqrt(-1);
    % Compute cepstrum by adding diagonals
    for i=1:np
        cuso(i)=0;
        for j=1:nle-i;
            cuso(i)=cuso(i)+ca1(j,i+j);
        end;cuso(i)=cuso(i)/(nle);
    end;
    cep=(1:1:np).*cuso(1:np);clear cuso auso;
    % Compute impulse response
    i=2;h(1)=1;
    while i<np+0.5;
       h(i)=0;
       for k=1:i-1;
          h(i)=h(i)+h(k)*cep(i-k);
       end;
       h(i)=h(i)/(i-1);i=i+1;
     end;
     figure(7);plot(h(2:np),'b');hold on;plot(himp(2:np),'r');hold on;
     % Write the system Haenkel matrix
     hh=zeros(np,nq);
     for i=1:np;
       for k=1:nq;
         if k<=i;
            hh(i,k)=h(i-k+1);
         end;
       end;
     end;
     [u d]=eig(ra1,hh'*hh);
     sas=diag(d);ua=[];sasn=[];
     for j=1:nq;
       dmin=10e6;
       for i=1:nq;
         if abs(sas(i))<dmin;
           dmin=abs(sas(i));nqq=i;
         end;
       end;
       sasn=[sasn;sas(nqq)];sas(nqq)=10e9;ua=[ua u(:,nqq)];
     end;
     % Denominator coefficients
     outd=real(ua(:,1)');
     %% For overestimated order
     %%iover=0;
     %%if iover==1;
     %%  sad=diff(sasn);ila=1;
     %%  for i=1:length(sad);
     %%    if sad(i)<1.5;
     %%        outd=outd+ua(:,i+1)';ila=ila+1;
     %%    else;
     %%        break;
     %%    end;
     %%  end;
     %%end;
     %%
     outd=outd/outd(1);sigma=1/sqrt(sasn(1));
     hha=hh/sigma;
     % Numerator coefficients
     outn=outd*hha';
     % Normalization
     sigma=sigma*outn(1);outn=outn/outn(1);
     %%pause;
%---------------------------------------------------------------------